// Virtuino client code to communicate with Virtuino server or Virtuino Emulator server 
// Code for all ESP8266 modules
// Created by Ilias Lamprou
// 24/6/2017

#include <ESP8266WiFi.h>

#define DIGITAL_PIN 0
#define DV_PIN 1
#define PWM_PIN 2
#define V_PIN 3

//-----------------SETTINGS -------------------
const char* ssid     = "WIFI NETWORK";
const char* password = "1111111111";

const char* virtuinoServerIP = "192.168.1.3";
int virtuinoServerPort = 8001;
String virtuinoServerPassword="1234";
//---------------------------------------------

String commandBuffer="";



//==================================================== setup
//====================================================
void setup() {
  Serial.begin(9600);
  delay(10);
  Serial.println();
  Serial.print("Connecting to ");
  Serial.println(ssid);
  WiFi.begin(ssid, password);
  while (WiFi.status() != WL_CONNECTED) {
    delay(500);
    Serial.print(".");
  }
  Serial.println("");
  Serial.println("WiFi connected");  
  Serial.println("IP address: ");
  Serial.println(WiFi.localIP());

  pinMode(D4, OUTPUT);
}



//==================================================== loop
//====================================================
void loop() {

//----------- Add code here to read your sensor values
   int value1 = analogRead(A0);           // Replace with the value you want to send to virtuino
   int value2 = random(100);              // Replace with the value you want to send to virtuino




//----------- Send comand to Virtuino   
   commandBuffer="";                      // clear buffer before send new command to Virtuino emulator web server
   
   addCommand(DIGITAL_PIN,0,"?");         // Command 1. Ask Virtuino app what is the value of emulator digital pin 0
   addCommand(PWM_PIN,3,"?");             // Command 1. Ask Virtuino app what is the value of emulator PWM pin 3
   
   addCommand(V_PIN,2,String(value1));    // Command 2. Send the value1. Add a value display to pin V2 to read this value
   addCommand(DV_PIN,15,String(value2));  // Command 3. Send the value2. add a value display to pin DV15 to read this value

 //-- more examples
 //addCommand(DV_PIN,8,"?");              // Ask Virtuino app what is the value of emulator digital pin DV8
 //addCommand(V_PIN,12,"?");              // Ask Virtuino app what is the value of emulator digital pin V12
 //addCommand(DIGITAL_PIN,7,String(1));   // Set the emulator pin D7 to 1
 // ...... 

   if (sendCommand()){                    // send all commands to virtuino emulator. Check the response
   
    //-- check the value of digital pin D0
     byte valueOf_D0= getResponseValue(DIGITAL_PIN,0);   // Read the value of pin D0
     Serial.println("D0 = "+String(valueOf_D0));
     if (!isnan(valueOf_D0)) {
        if (valueOf_D0==1) digitalWrite(D4,LOW);
        else if (valueOf_D0==0) digitalWrite(D4,HIGH);
     }
     else Serial.println("Response value of D0 = null");   // We get an error message  

     //-- check the value of PWM pin D3
     byte valueOf_D3= getResponseValue(PWM_PIN,3);         // Read the value of pin D3
     if (!isnan(valueOf_D3)) Serial.println("D3 PWM = "+String(valueOf_D3));

   //-- more examples
   //-- check the value of pin DV8
   //  int valueOf_DV8= getResponseValue(DV_PIN,8);       // Read the value of pin DV8
   //  if (!isnan(valueOf_DV8)) Serial.println("DV8 = "+String(valueOf_DV8));
   //-- check the value of pin V12
   //  float valueOf_V12= getResponseValue(V_PIN,12);       // Read the value of pin V12
   //  if (!isnan(valueOf_V12)) Serial.println("V12 = "+String(valueOf_V12));
   
   // ....
   }
   else {
    Serial.println("Error. Can't send command");
   }

}

//==================================================== sendCommand
//====================================================
boolean sendCommand(){
  if (commandBuffer.length()==0) return false;
  Serial.print("connecting to ");
  Serial.println(virtuinoServerIP);
  
  // Use WiFiClient class to create TCP connections
  WiFiClient client;
    if (!client.connect(virtuinoServerIP,virtuinoServerPort)) {
    Serial.println("connection failed");
    return false;
  }
  
  // We now create a URI for the request
  String virtuinoCommands=virtuinoServerPassword;
  virtuinoCommands+=commandBuffer;
  virtuinoCommands += "\r\n\r\n";
  
  // This will send the request to the server
  client.print(String("GET /") + virtuinoCommands+ " HTTP/1.1\r\n" +
               "Host: " + virtuinoServerIP + "\r\n" + 
               "Connection: close\r\n\r\n");
  unsigned long timeout = millis();
  while (client.available() == 0) {
    if (millis() - timeout > 5000) {
      Serial.println(">>> Client Timeout !");
      client.stop();
      return false;
    }
  }
  
  // Read all the lines of the reply from server and print them to Serial
  while(client.available()){
    String line = client.readStringUntil('\r');
    Serial.print(line);
    int commandStartIndex =  line.indexOf('!');
    int commandEndIndex =  line.lastIndexOf('$');
    if ((commandStartIndex!=-1) and (commandEndIndex!=-1) and (commandEndIndex-commandStartIndex>5)){
      commandBuffer= line.substring(commandStartIndex,commandEndIndex+1); 
      Serial.println("Virtuino Response: "+commandBuffer);
      client.flush();
    }
  }
  
  Serial.println();
  Serial.println("closing connection");
  return true;
  }

//======================================================================== addCommand
//========================================================================
void addCommand(byte pinType, byte pin , String value){
  char c=' ';
  if (pinType==DIGITAL_PIN) c='Q';
  else if (pinType==DV_PIN) c='D';
  else if (pinType==PWM_PIN) c='O';
  else if (pinType==V_PIN) c='V';
  
  if ((c=='Q') or (c=='D') or (c=='V') or (c=='O')){
    if ((pin>=0) and (pin <100)){
      String thisCommand="!";
      thisCommand+=c;
      if (pin<10) thisCommand+='0';
      thisCommand+=pin;
      thisCommand+='=';
      thisCommand+=value;
      thisCommand+='$';
      commandBuffer+=thisCommand;
      }
    }
  }

//======================================================================== getResponseValue
//========================================================================
float getResponseValue(byte pinType, byte pin){
  char c=' ';
  if (pinType==DIGITAL_PIN) c='Q';
  else if (pinType==DV_PIN) c='D';
  else if (pinType==PWM_PIN) c='O';
  else if (pinType==V_PIN) c='V';
  else return NULL;
  String header="!";
  header+=c;
  if (pin<10) header+="0";
  header+=pin;
  header+= "=";
  int pos= commandBuffer.indexOf(header);
  if (pos!=-1) {
    int lastChar = commandBuffer.indexOf('$',pos);
    if (lastChar!=-1){
      String valueString = commandBuffer.substring(pos+5,lastChar);
      valueString.trim();
      return valueString.toFloat();
    }
  }
  return NULL;
}

